from concierge_paas_plugin.settings import *

SECRET_KEY = "test-key"

DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'concierge_paas_plugin_test.db',
        }
    }


def createProfile(userId, name, email):
    return {'query': 'mutation{createProfile(gcId: "' + str(userId) + '", name: "' + name + '", email:"' +
                              email + '"){gcID, name, email}}'}

def disableProfile(userId):
    return {'query': 'mutation{ModifyProfile(gcId: "' + str(userId) +'"){gcID, name, email}}'}


import requests
from concierge_paas_plugin.helper import model_configuration

def executeQuery(query):
    retry = 3
    success = False
    attempt = 0

    configuration = model_configuration.getdefault()
    while not success and attempt < retry:
        response = requests.post(configuration.end_point, headers={'Authorization': 'Token ' + configuration.token}, data=query)
        if not response.status_code == requests.codes.ok:
            attempt = attempt + 1
        else:
            success = True

    if not success:
        return None

    response = response.json()
    return response


# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-09-24 13:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('concierge_paas_plugin', '0002_user'),
    ]

    operations = [
        migrations.DeleteModel(
            name='User',
        ),
    ]



from django.test import TransactionTestCase
from concierge_paas_plugin.api import create_profile, queryprofile, disableProfile
from concierge_paas_plugin.models import Configuration


class CreateProfileTest(TransactionTestCase):
    def setUp(self):
        self.configuration = Configuration(token='045d5617bbbd13c523be1fa6b86486e3c8a57b00', end_point="http://localhost:8001/protected", default=True, trigger=True)
        self.configuration.save()

        self.userId = 123456
        self.username = "custom_username"
        self.email = self.username + '@sometest.com'

        response = create_profile(self.userId, self.username, self.email)
        self.assertIsNotNone(response)

    def tearDown(self):
        disableProfile(self.userId)

    def test_createBasicProfile(self):
        response = queryprofile(self.userId)
        self.assertTrue(self.username in str(response))

    def test_userAlreadyExist(self):
        response = create_profile(self.userId, self.username, self.email)
        self.assertIsNone(response)



from django.contrib import admin
from .models import Configuration

# Register your models here.
class PaasAdmin(admin.ModelAdmin):
    list_display = ('end_point', 'trigger', 'default')
    list_editable = ['default']

admin.site.register(Configuration)


import os

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'concierge_paas_plugin.settings')

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'oidc_provider',
    'rest_framework',
    'rest_framework.authtoken',
    'concierge_paas_plugin',
    'user_sessions'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'user_sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ALLOWED_HOSTS = []
DEBUG =True

SECRET_KEY = os.getenv('SECRET_KEY')

ROOT_URLCONF = 'concierge_paas_plugin.urls'

USE_TZ = True


from django.test import TransactionTestCase
from concierge_paas_plugin.models import Configuration

class CreateProfileTest(TransactionTestCase):
    def setUp(self):
        self.configuration = Configuration(token='sometoken', end_point="http://localhost:8001/protected", default=True)
        self.configuration.save()
    def test_createBasicProfile(self):
        previously_createdConfiguration = Configuration.objects.get(default=True)
        self.assertEqual(previously_createdConfiguration.end_point, self.configuration.end_point)


from django.db import models
from django.db.models import Q

# Create your models here.
class Configuration(models.Model):
    end_point = models.URLField(blank=False, null=False)
    token = models.CharField(max_length=50, blank=False, null=False)
    trigger = models.BooleanField(default=False, null=False)
    default = models.BooleanField(default=False, null=False)

    def __str__(self):
        return self.end_point

    def save(self, *args, **kwargs):
        if(self.default == True):
            self.__class__.objects.filter(~Q(id=self.id)).update(default=False)
        super(Configuration, self).save(*args, **kwargs)


def QueryProfile(userId):
    return {'query': 'query{profiles(gcID: "' + str(userId) + '"){name, email, avatar, mobilePhone, officePhone,' +
                 'address{streetAddress,city, province, postalCode, country}}}'}

def QueryProfileByUserName(userName):
    return {'query': 'query{profiles(name: "' + str(userName) + '"){name, email, avatar, mobilePhone, officePhone,' +
                 'address{streetAddress,city, province, postalCode, country}}}'}

def QueryProfileByEmail(email):
    return {'query': 'query{profiles(email: "' + str(email) + '"){name, email, avatar, mobilePhone, officePhone,' +
                 'address{streetAddress,city, province, postalCode, country}}}'}


import requests
from oidc_provider.lib.claims import ScopeClaims
from .helper import graphql_query, graphql_mutation, graphql_post, model_configuration
from django.utils.translation import gettext as _
from django.db.models.signals import post_save

def create_profile(id, name, email):
    configuration = model_configuration.getdefault()

    # ToDo replace this with messages over Kafka system for notification
    if configuration.trigger is True:
        attempt = 0
        retrys = 3
        success = False

        if(name in queryProfileByName(name)):
            return None

        if(email in queryProfileByEmail(email)):
            return None

        query = graphql_mutation.createProfile(id, name, email)

        while not success and attempt < retrys:
            response = requests.post(configuration.end_point, headers={'Authorization': 'Token ' + configuration.token},
                                    data=query)
            if not response.status_code == requests.codes.ok:
                attempt = attempt + 1
            else:
                success = True
        if not success:
            # Write this to log eventually
            raise Exception('Error setting user data / Server Response ' + str(response.status_code))

post_save.connect(create_profile, sender=User)

def queryprofile(userId):
    # Get rest of information from Profile as a Service
    query = graphql_query.QueryProfile(userId)
    return graphql_post.executeQuery(query)

def queryProfileByName(userName):
    query = graphql_query.QueryProfileByUserName(userName)
    return graphql_post.executeQuery(query)

def queryProfileByEmail(email):
    query = graphql_query.QueryProfileByEmail(email)
    return graphql_post.executeQuery(query)

def disableProfile(userId):
    query = graphql_mutation.disableProfile(userId)
    return graphql_post.executeQuery(query)

def userinfo(claims, user):
    """
    Populate claims dict.
    """
    retrys = 3

    claims['name'] = user.name
    claims['email'] = user.email

    if model_configuration.getdefault().trigger:
        return claimsfromprofiles(retrys, claims)
    else:
        return claims

def claimsfromprofiles(retry, claims):
    response = queryprofile(retry)

    if response is None:
        return claims

    try:
        profileclaims = claims
        response_address = response['data']['profiles'][0].get('address')

        profileclaims['picture'] = checkvalue(response['data']['profiles'][0].get('avatar'))
        if response['data']['profiles'][0].get('mobilePhone') is not None:
            profileclaims['phone_number'] = checkvalue(response['data']['profiles'][0].get('mobilePhone'))
        else:
            profileclaims['phone_number'] = checkvalue(response['data']['profiles'][0].get('officePhone'))

        if response_address is not None:

            if 'streetAddress' in response_address:
                profileclaims['address']['street_address'] = checkvalue(response_address.get('streetAddress'))
            if 'city' in response_address:
                profileclaims['address']['locality'] = checkvalue(response_address.get('city'))
            if 'province' in response_address:
                profileclaims['address']['region'] = checkvalue(response_address.get('province'))
            if 'postalCode' in response_address:
                profileclaims['address']['postal_code'] = checkvalue(response_address.get('postalCode'))
            if 'country' in response_address:
                profileclaims['address']['country'] = checkvalue(response_address.get('country'))
    except Exception:
        return claims

    return profileclaims


def checkvalue(stringvalue):
    if stringvalue is not None:
        return stringvalue
    else:
        return ''


class CustomScopeClaims(ScopeClaims):
    info_modify_profile = (
        _('Profile Modification'),
        _('Ability to view and modify your profile information'),
    )

    def scope_modify_profile(self):
        # self.user - Django user instance.
        # self.userinfo - Dict returned by OIDC_USERINFO function.
        # self.scopes - List of scopes requested.
        # self.client - Client requesting this claims.
        dic = {
            'modify_profile': 'True',
            'detailed_profile': 'True'
        }

        return dic

    info_profile = (
        _('Basic profile'),
        _('Access to your name, email, avatar, and address'),
    )

    info_detailed_profile = (
        _('Detailed User profile'),
        _('Access to your name, email, avatar, address, and organization information'),
    )

    def scope_detailed_profile(self):
        dic = {
            'detailed_profile': 'True',
        }

        return dic


# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-09-21 12:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('concierge_paas_plugin', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('username', models.SlugField(unique=True)),
                ('name', models.CharField(max_length=100)),
                ('email', models.EmailField(max_length=255, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import os
from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='concierge_paas_plugin',
    version='0.10',
    packages=find_packages(),
    include_package_data=True,
    license='OSI Approved :: European Union Public Licence 1.1 (EUPL 1.1)',
    description='The Government of Canada paas for Concierge',
    url='https://github.com/gctools-outilsgc/concierge-paas-plugin',
    maintainer='GCTools Team',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=['django-oidc-provider==0.5.2']
)


import os
import django
from django.conf import settings

# We manually designate which settings we will be using in an environment variable
# This is similar to what occurs in the `manage.py`
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'concierge_paas_plugin.tests.testssettings')


# `pytest` automatically calls this function once when tests are run.
def pytest_configure():
    settings.DEBUG = True

    settings.PROJECT_DIR = os.path.abspath(os.path.dirname(__file__))

    settings.DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'concierge_paas_plugin_test.db',
        }
    }

    # If you have any test specific settings, you can declare them here,
    # e.g.
    # settings.PASSWORD_HASHERS = (
    #     'django.contrib.auth.hashers.MD5PasswordHasher',
    # )
    django.setup()
    # Note: In Django =< 1.6 you'll need to run this instead
    # settings.configure()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-09-20 10:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Configuration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('end_point', models.URLField()),
                ('token', models.CharField(max_length=50)),
                ('trigger', models.BooleanField(default=False)),
                ('default', models.BooleanField(default=False)),
            ],
        ),
        migrations.RunSQL([('INSERT INTO concierge_paas_plugin_configuration (trigger, end_point, token, \'default\') values (%s, %s, %s, %s)', [True, 'https://graphql.gccollab.ca/protected', 'aeb69b36cc2d332be878887971a61bf16411c3c0', True])])
    ]
